// Clock value: 11.0592 kHz


#include <inavr.h>
#include <iom8.h>
#include <pgmspace.h>

#include "fuel.h"
#include "uart_f.h"
#include "delay.h"
#include "eeprom_f.h"
#include "vstimer.h"
#include "bitmask.h"
#include "termmic.h"


#define FUEL_AVERAGING                16             //   
#define SAVE_PARAM_TIMEOUT_VALUE      50
#define FUEL_MEASURE_TIMEOUT_VALUE    500
#define TEMPER_MEASURE_TIMEOUT_VALUE  30

#define OPCODE_EEPROM_PARAM_SAVE      1
#define EEPROM_PARAM_START            0x002           //    EEPROM

#define TIMER2_RELOAD_VALUE           100             // 10 

#define GET_DEFEEPROM_JUMPER_STATE()  (PIND_Bit2)

#define TIMER_START                    DDRD &= ~(1<<6); TCCR1B = (1<<ICES1)|(1<<CS10)
#define TIMER_STOP                     TCCR1B = 0; DDRD |= (1<<6)
//----------- ----------------------------
s_timer16  fuel_measure_timeout_counter = 0;
s_timer8  temper_measure_timeout_counter = 0;
s_timer8 save_param_timeout_counter = 0;

extern ecudata edat;
ecudata *d=&edat;

unsigned char op_con;
unsigned char eeprom_parameters_cache[sizeof(fparams) + 1];
unsigned int overflow_timer;
unsigned long fuel_circular_buffer[FUEL_AVERAGING];//   

//*************************************************************

//   / 2 
//  10
#pragma vector=TIMER2_OVF_vect
__interrupt void timer2_ovf_isr(void)
{     
  TCNT2 = TIMER2_RELOAD_VALUE;  
  __enable_interrupt();  
  
  s_timer_update(save_param_timeout_counter);
  s_timer_update(fuel_measure_timeout_counter);
  s_timer_update(temper_measure_timeout_counter);
}
//___________________________________________________________________________________
//   / 1 
#pragma vector=TIMER1_OVF_vect
__interrupt void timer1_ovf_isr(void)
{
        overflow_timer++;   
}

//  
#pragma vector=ANA_COMP_vect
__interrupt void ANA_COMP_isr(void)
{  
    TIMER_STOP;
    op_con=0;
    d->dat_cap = TCNT1;      //     overflow_timer++;
    SETBIT(ACSR,ACD);       // 
    PORTC |=(1<<2);  
 
}

//
void measure_cap(void)
{ 
  op_con=1;
  overflow_timer=0;
  CLEARBIT(ACSR,ACD);   // 
  TCNT1=0;              // Timer1  
  PORTC &= ~(1<<2);     //
  
  TIMER_START;  
}

//*************************************************************
//   EEPROM,        
//    FLASH.
void load_eeprom_params(ecudata* d)
{     
 if (GET_DEFEEPROM_JUMPER_STATE())
   { 
   //   EEPROM
   eeprom_read(&d->fparam,EEPROM_PARAM_START,sizeof(fparams));    
   //  ,       
   // . 
   memcpy(eeprom_parameters_cache,&d->fparam,sizeof(fparams));  
   
   } 
 else
 { //  -   ,     
  memcpy_P(&d->fparam,&def_param,sizeof(fparams));
  d->op_actn_code =1;          
 }
 
} 
//   EEPROM -   .       ,
//                EEPROM.
//   EEPROM             
// UART-a      .        
void save_param_if_need(ecudata* d)
{  
  if (d->op_actn_code == OPCODE_EEPROM_PARAM_SAVE)
    goto force_parameters_save; //goto -  .
    
  //     ?
  if (s_timer16_is_action(save_param_timeout_counter)) 
  {
    //    ?
    if (memcmp(eeprom_parameters_cache,&d->fparam,sizeof(fparams)))
    {
      d->op_actn_code = 1;
force_parameters_save:    
    //     ,   EEPROM     -  
    //     EEPROM       .
    if (!eeprom_is_idle())
      return;

     memcpy(eeprom_parameters_cache,&d->fparam,sizeof(fparams));      
     eeprom_start_wr_data(OPCODE_EEPROM_PARAM_SAVE,EEPROM_PARAM_START,eeprom_parameters_cache,sizeof(fparams));        
     d->op_actn_code = 0; //       
    }
    s_timer16_set(save_param_timeout_counter, SAVE_PARAM_TIMEOUT_VALUE);
  }   
}
//*************************************************************
long measure(void)
{
  return (((unsigned long)overflow_timer<< 16) + d->dat_cap)>>2;
}
//   
void meas_update_values_buffers(ecudata* d)
{
   static unsigned char  fuel_ai  = FUEL_AVERAGING-1;   
    
  fuel_circular_buffer[fuel_ai] = measure();    
  (fuel_ai==0) ? (fuel_ai = FUEL_AVERAGING - 1): fuel_ai--; 
 
}

unsigned int fuel_to_v(unsigned int adcvalue, unsigned int offset, unsigned int gradient)
{   
 if (adcvalue < gradient)
   adcvalue = gradient;

 return (((unsigned long)(adcvalue - gradient)*(unsigned long)d->fparam.cap_tank) /((unsigned long)offset-gradient)) ; 
}
//   
void average_measured_values(ecudata* d)
{   
  unsigned char i;  unsigned long sum; 
  
  for (sum=0,i = 0; i < FUEL_AVERAGING; i++)  
  sum+=fuel_circular_buffer[i];     
  d->sens.fuel_raw =(sum/FUEL_AVERAGING); //     
  //    
  d->sens.fuel = fuel_to_v(d->sens.fuel_raw, d->fparam.Fmax, d->fparam.Fmin);  
}
//******************************************************************************
//  /
void init_io_ports(void)
{  
  PORTC= (1<<0)|(1<<1)|(1<<2);
  DDRC=0xff;
  PORTD  = (1<<PD2);
  DDRD |= (1<<6); //   
} 

void init_system_timer(void)
{ 
 TCCR2 = (1<<CS22)|(1<<CS21);       
//     2,  1  
 TIMSK =(1<<TOIE1)| (1<<TOIE2);
}

void init_comp(void)
{      
// ,      0  1,
//      
  ACSR =(1<<ACD)|(1<<ACIS0)|(1<<ACIS1)|(1<<ACIE);    
}
//*****************************************************************************
__C_task void main(void)
{
   
   edat.op_while=1;
  
   init_io_ports();   
   
   load_eeprom_params(&edat);//  eeprom
   
   uart_init(CBR_9600); 
   
   init_comp();
   
   init_system_timer();
  
 __enable_interrupt();   //   
   //   (,  )
   search_ROM();
   
   uart_send_packet(&edat,  RED_FUEL_PAR);//    
   
while (d->op_while)   
      {   
     //   EEPROM
      save_param_if_need(&edat); 
       if (s_timer_is_action(temper_measure_timeout_counter))
    { 
      
     d->temperature=read_temp(0); 
     d->temperature1=read_temp(1);
     s_timer_set(temper_measure_timeout_counter, TEMPER_MEASURE_TIMEOUT_VALUE);
     
    }       
      
      if (s_timer_is_action(fuel_measure_timeout_counter))
       {      
          if (op_con==0)
           {
             measure_cap();
             delay_ms(1);
            }
          else {op_con=0;TIMER_STOP;}
       //   
       meas_update_values_buffers(&edat);
       s_timer_set(fuel_measure_timeout_counter, FUEL_MEASURE_TIMEOUT_VALUE);
        }
     average_measured_values(&edat);//  
      };
}






