/*
       ds18b20

*/



#include <iom8.h>
#include "delay.h"
#include "termmic.h"

unsigned char ROM[2][8];

void OW_bit_0(void)
{
        OW_PORT &= ~(1<<OW_DQ);//  0  PORTB,  PB0 
        OW_DDR|=(1<<OW_DQ);    // 0   DDRB DQ -> 1  
        delay_us(60);          //60 
        OW_DDR&=~(1<<OW_DQ);        
        delay_us(10);          //10 
}

void OW_bit_1(void)
{
        OW_PORT &= ~(1<<OW_DQ); 
        OW_DDR|=(1<<OW_DQ);        
        delay_us(6);   //   6 
        OW_DDR&=~(1<<OW_DQ);         
}                

//  1-wire
unsigned char OW_reset(void)
{
  unsigned char answer;
  __disable_interrupt();
  OW_PORT &= ~(1<<OW_DQ); 
  OW_DDR |= (1<<OW_DQ);  
  delay_ms(0.48); // 0  .  480   
  
  OW_DDR &=~(1<<OW_DQ);   
  delay_us(70); //   70 
  answer = (OW_PIN & (1<<OW_DQ)); 
  delay_ms(0.41); //410  
   __enable_interrupt();
  return (answer); //: 0- , 1- 
 }

// 
void OW_write_bit(unsigned char bit)
{
  OW_PORT &= ~(1<<OW_DQ); //set 0 in PORTB, bit PB0
  OW_DDR |= (1<<OW_DQ); //set 0 in DDRB, line DQ -> 1  
  delay_us(10); 
    //  0  1   (bit == 0)
  if(bit) OW_DDR&=~(1<<OW_DQ);
  
  delay_us(70); 
  OW_DDR&=~(1<<OW_DQ);
}

// 
void OW_write_byte(unsigned char command)
 {
   __disable_interrupt();
    for (unsigned char i = 0; i < 8; i++)
     {
         OW_write_bit(command & 0x01); //   
         command >>= 1; //    
     }
    
    delay_us(120);
    __enable_interrupt();
    delay_ms(7);  
 }

//  
void convert_temp(void)
{
    OW_reset();              // 
    OW_write_byte(SKIP_ROM); //   ROM-(0xCC)
    OW_write_byte(CONVERT_T); //  T-(0x44)    
}

// 
 unsigned char OW_read_bit(void)
{  
  unsigned char result; 
  
  OW_bit_1();
  delay_us(10);   
  result = (OW_PIN & (1<<OW_DQ)); //  ( PB0 )
	
  return result;
}

// 
signed int read_temp(unsigned char t)
 {
  unsigned int value = 0; 
  unsigned char i;
  
    convert_temp();     
    send_ROM(t);
    OW_write_byte(READ_SCRATCHPAD); //  (0xBE)
    __disable_interrupt();
 //     1 Wire
  for(i = 0; i < 16; i++)
  {
    if(OW_read_bit()) value |= 0x01 << i;//         i
    
    delay_us(120); //     
  }
__enable_interrupt();  
  return(value);
 }
                
//  ROM  
void send_ROM(unsigned char selector)
{
  unsigned char j;
  OW_reset();              //     
  OW_write_byte(MATCH_ROM);// ROM
  
  for(j=0;j<8;j++)
  {  
    OW_write_byte(ROM[selector][j]);    
  }
  
}

//   
void search_ROM(void)
{
unsigned char get_for_two=0;
unsigned char i,j; 
unsigned char bit_analizer; //   

TWO:
OW_reset();            // 
OW_write_byte(SEARCH_ROM);  //  
delay_ms(0.1);
__disable_interrupt();
//   
  for(j=0;j<8;j++) 
  {  
   for(i=0;i<8;i++)     
   {
     //   
     bit_analizer=0;    
     if (OW_read_bit()) bit_analizer|=(1);       
     delay_us(55);
     //     
     if (OW_read_bit()) bit_analizer|=(0x01 <<1);     
     else
     { 
     bit_analizer&=~(0x01 <<1);
     }
     delay_us(55);  
   //   (  )
     switch(bit_analizer)
	 {
     case 0:          
      if(get_for_two==0) //  1-   . 0	                                    
	   {                    
	    ROM[0][j]&=~(0x01 <<i); 	
            OW_bit_0(); // . 0      
	    }
	else //  2-   . 1
	     {   
	      ROM[1][j]|=(0x01 <<i);                 
              OW_bit_1();    // .1  
              delay_us(64);  //   64 
	      }
	       break;
      case 1: 
       if(get_for_two==0) // .1
	 {
	    ROM[0][j]|=(0x01 <<i);
	 }
	else
	 {
	    ROM[1][j]|=(0x01 <<i); 	
	  }
             // . 1  
             OW_bit_1();
             delay_us(64);  //   64 
                break;
      case 2: 
       if(get_for_two==0)// . 0
	 {
	   ROM[0][j]&=~(0x01 <<i);
	 }
        else
         {
	   ROM[1][j]&=~(0x01 <<i); 
	 }
	     // .0  
             OW_bit_0();   
		break;
       case 3:break;
       };       	  
     }
   }
if(get_for_two==0 ) 
{
get_for_two=1;
delay_ms(7);
goto TWO;
}

__enable_interrupt();
  
}

void OW_read_ROM(unsigned char get_for_two)
{
unsigned char j;

OW_reset();
OW_write_byte(READ_ROM );
__disable_interrupt();
 //     1 Wire
  for(j = 0; j < 8; j++)
  { 
  for(unsigned char i  = 0; i < 8; i++)
  {
    if(OW_read_bit()) ROM[get_for_two][j] |= 0x01 << i;//         i
   
    delay_us(120); //     
  }
   }
  __enable_interrupt();
}  



