 /****************************************************************
 *       UART
 ****************************************************************/

#include <ina90.h>
#include <iom8.h>
#include <pgmspace.h>
#include "uart_f.h"
#include "bitmask.h"
#include "delay.h"

ecudata edat;

typedef struct
{ 
 char send_mode; 
 unsigned char recv_buf[UART_RECV_BUFF_SIZE];// 
 unsigned char send_buf[UART_SEND_BUFF_SIZE];// 
 unsigned char send_size;
 unsigned char send_index;
 unsigned char recv_size;
 unsigned char recv_index;
}UARTSTATE;

UARTSTATE uart;

 

const __flash char hdig[] = "0123456789ABCDEF";


#define HTOD(h) (((h)<0x3A) ? ((h)-'0') : ((h)-'A'+10))

//--------    -------------


#define build_i4h(i) {uart.send_buf[uart.send_size++] = ((i)+0x30);}

void build_i8h(unsigned char i)
{
 uart.send_buf[uart.send_size++] = hdig[i/16];    //  HEX 
 uart.send_buf[uart.send_size++] = hdig[i%16];    //  HEX 
}

void build_i16h(unsigned int i)
{
 uart.send_buf[uart.send_size++] = hdig[GETBYTE(i,1)/16];    //  HEX  ( )
 uart.send_buf[uart.send_size++] = hdig[GETBYTE(i,1)%16];    //  HEX  ( )
 uart.send_buf[uart.send_size++] = hdig[GETBYTE(i,0)/16];    //  HEX  ( )
 uart.send_buf[uart.send_size++] = hdig[GETBYTE(i,0)%16];    //  HEX  ( )
}


void build_i32h(unsigned long i)
{
 build_i16h(i>>16);
 build_i16h(i);
}

//----------    ---------
#define recept_i4h() (uart.recv_buf[uart.recv_index++] - 0x30)
#define recept_name()(uart.recv_buf[uart.recv_index++] )

unsigned char recept_i8h(void)
{
 unsigned char i8;    
 i8 = HTOD(uart.recv_buf[uart.recv_index])<<4;
 uart.recv_index++;
 i8|= HTOD(uart.recv_buf[uart.recv_index]);
 uart.recv_index++;
 return i8;
}

unsigned int recept_i16h(void)
{
 unsigned int i16;    
 SETBYTE(i16,1) = (HTOD(uart.recv_buf[uart.recv_index]))<<4;          
 uart.recv_index++;
 SETBYTE(i16,1)|= (HTOD(uart.recv_buf[uart.recv_index]));          
 uart.recv_index++;
 SETBYTE(i16,0) = (HTOD(uart.recv_buf[uart.recv_index]))<<4;    
 uart.recv_index++;
 SETBYTE(i16,0)|= (HTOD(uart.recv_buf[uart.recv_index]));          
 uart.recv_index++;
 return i16;
}


unsigned long recept_i32h(void)
{
 unsigned long i = 0;
 i = recept_i16h();
 i = i << 16;
 i|=recept_i16h();
 return i;
}

//--------------------------------------------------------------------


void uart_begin_send(void)
{
 uart.send_index = 0;
 UCSRB |= (1<<UDRIE); /*    .  */ 
}


void uart_send_packet(ecudata* d, char send_mode)  
{
 
 //     ,       
 uart.send_size = 0; 
 
 if (send_mode==0) //  
   send_mode = uart.send_mode;
 
 uart.send_buf[uart.send_size++] = '!';
 uart.send_buf[uart.send_size++] = send_mode;
 
 switch(send_mode)
  {
    case FUEL_DAT:    
 build_i16h(d->sens.fuel_raw);
 build_i16h(d->sens.fuel);
 build_i16h(d->temperature);
 build_i16h(d->temperature1);
    break; 
    
    case RED_FUEL_PAR:
 build_i16h(d->fparam.Fmax);
 build_i16h(d->fparam.Fmin);
 build_i16h(d->fparam.cap_tank);
   break;                   
  }
  uart.send_buf[uart.send_size++] = '\r';
  //      -  
  uart_begin_send();
}
//------------------------------------------------------------------------------------------------
void uart_recept_packet(ecudata* d)
{ 
 unsigned char descriptor;   
 uart.recv_index = 0;
 descriptor = uart.recv_buf[uart.recv_index++]; 
 
switch(descriptor)
  {
    case FUEL_DAT:
    break;
    
    case RED_FUEL_PAR:
    break;
    
    case FUEL_PAR:  // 
 d->fparam.Fmax= recept_i16h();
 d->fparam.Fmin= recept_i16h();
 d->fparam.cap_tank= recept_i16h();
   break;                   
  }
 uart.send_mode = descriptor;
}

//uart  
void uart_notify_processed(void)
{
  uart.recv_size = 0;
  UCSRB &=~(1<<RXEN);
}
//uart -  
unsigned char uart_is_sender_busy(void)
{
  return (uart.send_size > 0);
}
//uart -  
unsigned char uart_is_packet_received(void)
{
  return (uart.recv_size > 0);
}

void uart_init(unsigned int baud)
{
  // Set baud rate  
  UBRRH = (unsigned char)(baud>>8);
  UBRRL = (unsigned char)baud;
  UCSRA = 0;                                                  //   
  UCSRB = (1<<RXCIE)|(1<<RXEN)|(1<<TXEN);                     //,      
  UCSRC=(1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0);                     //8 , 1 ,                                         
  uart.send_size = 0;                                         //    
  uart.recv_size = 0;                                         //     
}

void process_uart_interface(ecudata* d)
{ 
   
  if (uart_is_packet_received())//   ?
 { 
  uart_recept_packet(d);   
  //    -     
  uart_notify_processed();         
 }
  if (!uart_is_sender_busy())// ?
  {   
   uart_send_packet(d, uart.send_mode);       //         
  }
  UCSRB |=(1<<RXEN);
 
}

//      UART (   )
#pragma vector=USART_UDRE_vect
__interrupt void usart_udre_isr(void)
{ 
 if (uart.send_size > 0)
 {
  UDR = uart.send_buf[uart.send_index];
  uart.send_size--;
  uart.send_index++;
 }
 else
 {//  
  UCSRB &= ~(1<<UDRIE); // disable UDRE interrupt 
 }
}

#pragma vector=USART_RXC_vect
__interrupt void usart_rx_isr()
{
  static unsigned char state=0;
  unsigned char chr;
  edat.op_while = 0;
  
  //__enable_interrupt();
  chr = UDR; 
  switch(state)
  {
    case 0:            // (   )
      if (uart.recv_size!=0) //     ,     .
       break;       

      if (chr=='@')   // ?   
      {      
       state = 1;
       uart.recv_index = 0;      
      }
      break;

    case 1:           //                 
      if (chr=='\r')
      {             
       state = 0;       //         
       uart.recv_size = uart.recv_index; // ,   
       //***********************************************************************
        
       delay_ms(1);
       
       process_uart_interface(&edat);///  UART   
      }
      else
      { 
       if (uart.recv_index >= UART_RECV_BUFF_SIZE)
       {
       //: ! -    ,    !
       state = 0;               
       }
       else
        uart.recv_buf[uart.recv_index++] = chr;    
      }    
      break;                   
  }
  edat.op_while=1; 
}

 
